-- AutomaticBeaconLights v1.0.0.1
-- Release Date: March 20, 2025
-- Author: Merq
-- Description: Automatically controls beacon and hazard lights based on vehicle state and AI usage.

-- Configuration
-- Customize the behavior of beacon lights by setting BeaconControl to 1, 2, or 3:
-- 1: Automatic beacons for player actions only (entering, driving, exiting with engine on). AI behavior unchanged.
-- 2: Automatic beacons for AI workers only (on when AI controls vehicle, off when AI stops or engine off). Player behavior unchanged.
-- 3: (Default) Full control - beacons on for both player and AI when vehicle is occupied or engine is running.
local BeaconControl = 3  -- Default setting. Change to 1 or 2 to adjust behavior.

-- Enable or disable hazard lights for AI workers in vehicles without beacon lights:
-- 0: Disabled - AI workers will not toggle hazard lights on vehicles without beacons.
-- 1: (Default) Enabled - AI workers will toggle hazard lights on vehicles without beacons during jobs or driving.
local AIHazardsWithoutBeacons = 1  -- Default setting. Change to 0 to disable.

-- Enable or disable detailed debug logging:
-- 0: (Default) Disabled - no log output from this mod.
-- 1: Enabled - full debug output including state changes and events (useful for troubleshooting).
local DebuggingEnabled = 0  -- Debugging disabled for release

AutomaticBeaconLights = {}

function AutomaticBeaconLights.prerequisitesPresent(specializations)
    return true
end

function AutomaticBeaconLights.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onEnterVehicle", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", AutomaticBeaconLights)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", AutomaticBeaconLights)
end

function AutomaticBeaconLights:onLoad()
    self.beaconsActive = false
    self.hazardsActive = false
    self.wasAIActive = false
    self.wasPlayerActive = false
    self.wasUnderAIControl = false
    self.isAIWorking = false
    self.isValid = true
    self.manualBeaconOverride = false
    self.manualBeaconState = nil
    self.manualHazardOverride = false
    self.manualHazardState = nil
    self.hasBeaconLights = false
end

function AutomaticBeaconLights:onDelete()
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onDelete called") end
    self.isValid = false
end

function AutomaticBeaconLights:onUpdate(dt)
    if not self.isValid or self.getIsActive == nil then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping update - Vehicle is invalid or deleted") end
        return
    end

    local isActive = self:getIsActive()
    local isPlayerEntered = self:getIsEntered()
    local isAIActive = self:getIsAIActive()
    local isMotorStarted = self.getIsMotorStarted and self:getIsMotorStarted() or false

    if isAIActive or (self.wasUnderAIControl and isMotorStarted) then
        self.isAIWorking = true
    elseif not isMotorStarted then
        self.isAIWorking = false
    end

    if isAIActive then 
        self.wasAIActive = true 
        self.wasUnderAIControl = true 
    end
    if isPlayerEntered then 
        self.wasPlayerActive = true 
    end

    local shouldBeaconsBeOn = false
    if BeaconControl == 1 then
        shouldBeaconsBeOn = isPlayerEntered or (self.wasPlayerActive and isMotorStarted)
    elseif BeaconControl == 2 then
        shouldBeaconsBeOn = isAIActive or (self.isAIWorking and isMotorStarted)
    elseif BeaconControl == 3 then
        shouldBeaconsBeOn = isMotorStarted or isPlayerEntered or isAIActive or self.isAIWorking
    end

    -- Check if the vehicle has beacon lights (only do this once per update cycle)
    local spec = self.spec_lights
    if spec then
        self.hasBeaconLights = spec.beaconLights ~= nil and next(spec.beaconLights) ~= nil
    else
        self.hasBeaconLights = false
    end

    -- Check for manual toggle of beacons every frame
    local gameBeaconState = self:getBeaconLightsVisibility() or false
    if gameBeaconState ~= self.beaconsActive then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Detected manual beacon change - GameState: " .. tostring(gameBeaconState) .. ", ModState: " .. tostring(self.beaconsActive)) end
        self.manualBeaconOverride = true
        self.manualBeaconState = gameBeaconState
        self.beaconsActive = gameBeaconState
    end

    -- Check for manual toggle of hazard lights every frame
    local gameHazardState = self:getTurnLightState() or Lights.TURNLIGHT_OFF
    local isHazardOn = (gameHazardState == Lights.TURNLIGHT_HAZARD)
    if isHazardOn ~= self.hazardsActive then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Detected manual hazard change - GameState: " .. tostring(gameHazardState) .. ", ModState: " .. tostring(self.hazardsActive)) end
        self.manualHazardOverride = true
        self.manualHazardState = isHazardOn
        self.hazardsActive = isHazardOn
    end

    if DebuggingEnabled == 1 then
        print("AutomaticBeaconLights: State - Active: " .. tostring(isActive) .. ", Player: " .. tostring(isPlayerEntered) .. ", AI: " .. tostring(isAIActive) .. ", WasAI: " .. tostring(self.wasAIActive) .. ", WasPlayer: " .. tostring(self.wasPlayerActive) .. ", MotorStarted: " .. tostring(isMotorStarted) .. ", ShouldBeOn: " .. tostring(shouldBeaconsBeOn) .. ", BeaconsActive: " .. tostring(self.beaconsActive) .. ", HazardsActive: " .. tostring(self.hazardsActive) .. ", HasBeaconLights: " .. tostring(self.hasBeaconLights) .. ", ManualBeaconOverride: " .. tostring(self.manualBeaconOverride) .. ", ManualHazardOverride: " .. tostring(self.manualHazardOverride) .. ", GameBeaconState: " .. tostring(gameBeaconState) .. ", GameHazardState: " .. tostring(gameHazardState))
    end

    -- Handle beacon lights
    if self.manualBeaconOverride then
        if self.manualBeaconState ~= nil then
            self:setBeaconLightsVisibility(self.manualBeaconState)
            self.beaconsActive = self.manualBeaconState
        end
    else
        if shouldBeaconsBeOn then
            self:setBeaconLightsVisibility(true)
            self.beaconsActive = true
            if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Forcing beacons on") end
        elseif not shouldBeaconsBeOn and self.beaconsActive then
            self:setBeaconLightsVisibility(false)
            self.beaconsActive = false
            self.wasAIActive = false
            self.wasPlayerActive = false
            self.wasUnderAIControl = false
            self.isAIWorking = false
            if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Deactivating beacons") end
        end
    end

    -- Handle hazard lights
    if self.manualHazardOverride then
        if self.manualHazardState ~= nil then
            self:setTurnLightState(self.manualHazardState and Lights.TURNLIGHT_HAZARD or Lights.TURNLIGHT_OFF)
            self.hazardsActive = self.manualHazardState
        end
    else
        if AIHazardsWithoutBeacons == 1 then
            -- Only enable hazards if the vehicle is under AI control and has no beacon lights
            local shouldHazardsBeOn = isAIActive and not self.hasBeaconLights
            if shouldHazardsBeOn then
                self:setTurnLightState(Lights.TURNLIGHT_HAZARD)
                self.hazardsActive = true
                if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Forcing hazards on (AI active, no beacon lights)") end
            elseif not shouldHazardsBeOn and self.hazardsActive then
                self:setTurnLightState(Lights.TURNLIGHT_OFF)
                self.hazardsActive = false
                self.wasAIActive = false
                self.wasPlayerActive = false
                self.wasUnderAIControl = false
                self.isAIWorking = false
                if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Deactivating hazards") end
            end
        end
    end
end

function AutomaticBeaconLights:onEnterVehicle()
    if not self.isValid then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping onEnterVehicle - Vehicle is invalid") end
        return
    end
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onEnterVehicle called") end
    if self.wasUnderAIControl and not self:getIsAIActive() then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting post-AI control") end
        self.beaconsActive = false
        self.hazardsActive = false
        self.wasUnderAIControl = false
        self.isAIWorking = false
        self.wasAIActive = false
    end
    if self.manualBeaconOverride then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting manual beacon override on vehicle entry") end
        self.manualBeaconOverride = false
        self.manualBeaconState = nil
    end
    if self.manualHazardOverride then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting manual hazard override on vehicle entry") end
        self.manualHazardOverride = false
        self.manualHazardState = nil
    end
end

function AutomaticBeaconLights:onLeaveVehicle()
    if not self.isValid then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Skipping onLeaveVehicle - Vehicle is invalid") end
        return
    end
    if DebuggingEnabled == 1 then print("AutomaticBeaconLights: onLeaveVehicle called") end
    if self.manualBeaconOverride and not self.manualBeaconState and self:getIsMotorStarted() then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting manual beacon override on exit with engine running") end
        self.manualBeaconOverride = false
        self.manualBeaconState = nil
    end
    if self.manualHazardOverride and not self.manualHazardState and self:getIsMotorStarted() then
        if DebuggingEnabled == 1 then print("AutomaticBeaconLights: Resetting manual hazard override on exit with engine running") end
        self.manualHazardOverride = false
        self.manualHazardState = nil
    end
end